

--[[ 07-30-2022 (mm-dd-yyyy) version 0.8.0]]--



function JointMortorUnified(joint, speed, direction)
    local min, max = GetJointLimits(joint)
    if GetJointType(joint) == "hinge" then
        if direction == "max" then
            SetJointMotorTarget(joint, max, speed)
        else
            SetJointMotorTarget(joint, min, speed)
        end
    elseif GetJointType(joint) == "prismatic" then
        if direction == "max" then
            SetJointMotorTarget(joint, max, speed/math.pi)
        else
            SetJointMotorTarget(joint, min, speed/math.pi)
        end
    end
end


function ControlLegs(condition, legs, group)
	group = group or 1
	for i=1, #legs do
		local locallegs = GetBodyShapes(legs[i])
		local legGroup = tonumber(GetTagValue(legs[i], "Tleg")) or 1
		if legGroup == group then
			for t=1, #locallegs do
				local legJoints = GetShapeJoints(locallegs[t])
				if HasTag(legs[i], "Tactive") or IsBodyActive(legs[i]) then
					if condition then
						local min, max = GetJointLimits(legJoints[1])
						if GetJointMovement(legJoints[1]) <= max then
							SetTag(legs[i], "Tactive")
							for j=1, #legJoints do
								JointMortorUnified(legJoints[j], 1.5, "max")
							end
						else
							RemoveTag(legs[i], "Tactive")
						end
					else
						local min, max = GetJointLimits(legJoints[1])
						if GetJointMovement(legJoints[1]) >= min then
							SetTag(legs[i], "Tactive")
							for j=1, #legJoints do
								JointMortorUnified(legJoints[j], 1.5, "min")
							end
						else
							RemoveTag(legs[i], "Tactive")
						end
					end
				end
			end
		end
	end
end


function GetShapeCenter(shape)
	local min, max = GetShapeBounds(shape)
	local center = VecLerp(min, max, 0.5)
	return center
end


function ShapePosOffset(shape)
	local voxX, voxY, voxZ, voxScale = GetShapeSize(shape)
	local shapeTrans = GetShapeWorldTransform(shape)
	local centreVec = VecScale(Vec(voxX/2, voxY/2, 0), voxScale)
	local worldFrontVec = TransformToParentVec(shapeTrans, centreVec)
	return VecAdd(shapeTrans.pos, worldFrontVec)
end


function DecToHex(decNum)
    local hexString = "0123456789ABCDEF"
    local pos0, pos1 = (decNum%16)+1, math.floor(decNum/16)+1
    return string.sub(hexString, pos1, pos1)..string.sub(hexString, pos0, pos0)
end


function HexToDec(hexNum)
    local hexString = "0123456789ABCDEF"
    local pos1 = string.find(hexString, string.sub(hexNum, 1, 1))-1 or 0
    local pos0 = string.find(hexString, string.sub(hexNum, -1))-1 or 0
    return pos1*16+pos0
end


function HEXtoRGB(hexColor)
	local rgbStr = ""
	for a=1, 3 do
		if a ~= 1 then
			rgbStr = rgbStr.." "
		end
		rgbStr = rgbStr..HexToDec(string.sub(hexColor, a*2-1, a*2))/255
	end
	return rgbStr
end


function progressBar(w, h, t, RGBs)
	UiPush()
		UiAlign("left top")
		UiColor(0, 0, 0, 0.5)
		UiImageBox("ui/common/box-solid-10.png", w, h, 6, 6)
		if t > 0 then
			UiTranslate(2, 2)
			w = (w-4)*t
			if w < 12 then w = 12 end
			h = h-4
			UiColor(RGBs[1], RGBs[2], RGBs[3], RGBs[4])
			UiImageBox("ui/common/box-solid-6.png", w, h, 6, 6)
		end
	UiPop()
end


function DriveTrailer(trailer, vehicle, followVehi)
	local drive, steer, brake = 0, 0, false
	local localVel = TransformToLocalVec(GetVehicleTransform(vehicle), GetBodyVelocity(GetVehicleBody(vehicle)))
	local speed = -localVel[3]

	local frontVec = Vec(0, 0, 1)
	local upVec = Vec(0, 1, 0)
	local locFrontTrailer = TransformToParentVec(GetBodyTransform(GetVehicleBody(trailer)), frontVec)
	local vehicleTrailerVec = TransformToLocalVec(GetBodyTransform(GetVehicleBody(followVehi)), locFrontTrailer)

	local vx, vy, vz = GetQuatEuler(QuatAlignXZ(frontVec, upVec))
	local tx, ty, tz = GetQuatEuler(QuatAlignXZ(vehicleTrailerVec, upVec))

	drive = ((InputDown("up") and speed > 0.01) and 1) or ((InputDown("down") and speed < -0.01) and -1) or 0
	steer = tonumber(string.format("%.2f", (vy-ty)/30)) or 0
	brake = (InputDown("up") and speed < -0.01) or (InputDown("down") and speed > 0.01) or InputDown("handbrake")

	DriveVehicle(trailer, drive, steer, brake)
end


function ReturnToEnter(str)
	return ((string.lower(str) == "return") and "Enter") or str
end